#include "label.h"
#include <uta/multilineedit.h>
#include <uta/resources.h>
#include <uta/pushbutton.h>


using namespace Janus;

Label::Label(const Parameters& param) :
  WidgetBase(param),
  terminal_(NULL),
  slider_(NULL)
{
  assert(parent_);

  step_ = Element::getIntAttr("step",2);

  std::string scrollbar = Element::getStringAttr("scrollbar");

  widget_ = new uta::Widget(parent_, rect_.upperLeft().x, rect_.upperLeft().y,
			    rect_.width(), rect_.height());

  widget_->setAutoDelete(false);
  widget_->setColor(color_);
 
  if(hasAttr("background"))
    widget_->setBackground(background_);

  if(scrollbar.empty() || rect_.height() < 64)
    {
      terminal_ = new uta::Terminal(widget_, 
				    0, 0,
				    rect_.width(), rect_.height(), font_);
      terminal_->setMaxLinesToStore(1024);
      terminal_->scrolled.connect(slot(this, &Label::slot_terminal));
    }
  else
    {
      terminal_ = new uta::Terminal(widget_, 
				    0,0,
				    rect_.width()-16, rect_.height(), font_);   
      terminal_->setMaxLinesToStore(1024);
      terminal_->scrolled.connect(slot(this, &Label::slot_terminal));


      slider_ = new uta::Slider(widget_,
				rect_.width()-16, 16,
				16, rect_.height()-32, uta::Slider::VERTICAL);
  
      slider_->setBackground(NULL);
      slider_->setButtonColor(uta::lightgrey);
      slider_->setRange(0,terminal_->linesCount());
      slider_->setValue(terminal_->currentLine());
      slider_->valueChanged.connect(slot(terminal_, &uta::Terminal::scrollTo));
      if(hasAttr("slider"))
	slider_->setButtonSurface(SURFACE_RES(getAttr("slider")));
      if(hasAttr("slider_background"))
	slider_->setBackground(SURFACE_RES(getAttr("slider_background")));

      uta::PushButton* but;
      but = new uta::PushButton(widget_, rect_.width()-16,0, 16, 16);
      but->setRepeat(true);
      but->clicked.connect(SigC::slot(this, &Label::slot_scrollUp));
      if(hasAttr("arrow_up"))
	but->setUpPic(SURFACE_RES(getAttr("arrow_up")));
      if(hasAttr("arrow_up_pressed"))
	but->setDownPic(SURFACE_RES(getAttr("arrow_up_pressed")));
			   
      but = new uta::PushButton(widget_, rect_.width()-16,rect_.height()-16, 16, 16);
      but->setRepeat(true);
      but->clicked.connect(SigC::slot(this, &Label::slot_scrollDown));
      if(hasAttr("arrow_down"))
	but->setUpPic(SURFACE_RES(getAttr("arrow_down")));
      if(hasAttr("arrow_down_pressed"))
	but->setDownPic(SURFACE_RES(getAttr("arrow_down_pressed")));
      
    }

  if(hasAttr("text_background"))
    terminal_->setBackground(SURFACE_RES(getAttr("text_background")));
  else
    terminal_->setBackground(NULL);

  terminal_->setWrapping(true);

  std::string col = Element::getStringAttr("text_color","");
  std::string fade_col = Element::getStringAttr("fade_color",""); 

  if(RES_AVAILABLE(col) && RES_AVAILABLE(fade_col))
    terminal_->setTextColor(COLOR_RES(col), COLOR_RES(fade_col));
  else if(RES_AVAILABLE(col))
    terminal_->setTextColor(COLOR_RES(col));

  for(int n=1; n<8; n++)
    {
      std::string nr = std::string(1,char('0'+n));
      std::string col = Element::getStringAttr("text_color"+nr,"");

      if(RES_AVAILABLE(col) && RES_AVAILABLE(fade_col))
	terminal_->setMarkupColor(n, COLOR_RES(col), COLOR_RES(fade_col));
      else if(RES_AVAILABLE(col))
	terminal_->setMarkupColor(n, COLOR_RES(col));
    }
  for(int n=1; n<8; n++)
    {
      std::string nr = std::string(1,char('0'+n));
      std::string font = Element::getStringAttr("font"+nr,"");
      

      terminal_->setMarkupFont(n, FONT_RES(font));
    }
  for(int n=1; n<8; n++)
    {
      std::string nr = std::string(1,char('0'+n));
      std::string image = Element::getStringAttr("image"+nr,"");
      terminal_->setImage(n, SURFACE_RES(image));
    }

  terminal_->linkActivated.connect(SigC::slot(this, &Label::slot_link));

  terminal_->setText(Element::getAttr("text").c_str());    
  terminal_->scroll2Top();
  
  if(slider_)
    slider_->setDelta(terminal_->visibleLines() / 10 + 1);

}

Label::~Label()
{
  // libuta will auto delete slider and terminal if existing
}

void 
Label::call(const std::string& name, const Value& val)
{
  // dynamic_cast<uta::Label*> returns non-valid pointers !!!
  //  uta::MultiLineEdit* label = (uta::MultiLineEdit*)widget_;

  if(name == "setText")
    {
      terminal_->setText(val.c_str());
    }
  else if(name == "addText")
    {
      terminal_->addText(val.c_str());
    }
  else if(name == "scrollUp")
    {
      slot_scrollUp();
    }
  else if(name == "scrollDown")
    {
      slot_scrollDown();
    }
  else if(name == "scroll2Top")
    {
      terminal_->scroll2Top();
    }
  else if(name == "scroll2Bottom")
    {
      terminal_->scroll2Bottom();
    }
  else
    {
      WidgetBase::call(name, val);
    }
}

void
Label::slot_link(unsigned id)
{
  char buffer[32];
  sprintf(buffer,"%d",id);
  lastLink_ = buffer;
  ElementSigC::emit("linkActivated", buffer);
}

Value 
Label::getValue(void) const
{
    return lastLink_;
}

void 
Label::slot_terminal()
{
  if(slider_)
    {
      slider_->setDelta(terminal_->visibleLines()/10 +1);
      slider_->setRange(0,terminal_->linesCount());
      slider_->setValue(terminal_->currentLine());   
    }
}

void
Label::slot_scrollUp()
{
  terminal_->scrollUp(step_);
}

void
Label::slot_scrollDown()
{
  terminal_->scrollDown(step_);
}
